//
//  MSNearbyTableViewController.swift
//  MeridianSwiftSamples
//
//  Created by Stephen Kelly on 5/23/17.
//  Copyright © 2017 Aruba Networks. All rights reserved.
//

import UIKit

class MSNearbyTableViewController: UITableViewController, UISearchBarDelegate, MRLocationManagerDelegate, MSExampleViewController {
    private lazy var searchBar: UISearchBar = {
        let searchBar = UISearchBar(frame: CGRect(x: 0, y: 0, width: view.frame.size.width, height: 44))
        searchBar.delegate = self
        searchBar.placeholder = "Find up to ten local items"
        return searchBar
    }()

    private lazy var locationManager: MRLocationManager = {
        let locationManager = MRLocationManager(app: MREditorKey(identifier: MSExampleHost.appID()))
        locationManager.delegate = self
        return locationManager
    }()

    private var search: MRLocalSearch?
    private var autoUpdateTimer: Timer?
    private var searchResults = [MRPlacemarkResult]()

    override func viewDidLoad() {
        super.viewDidLoad()
        navigationItem.titleView = searchBar
        refreshControl?.addTarget(self, action: #selector(updateNearby), for: .valueChanged)
    }

    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)

        // start listening for location updates
        locationManager.startUpdatingLocation()
        updateNearby()
    }

    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        locationManager.stopUpdatingLocation()
    }

    // MARK: - MSExampleViewController

    static func exampleTitle() -> String {
        return "Local Search"
    }

    static func exampleDetail() -> String {
        return "Example of how to use the MRLocalSearch API to search for Placemarks based on the current location and display the results."
    }

    // MARK: - MRLocationManagerDelegate

    func locationManager(_ manager: MRLocationManager, didUpdateTo location: MRLocation) {
        guard let localSearch = search else {
            return
        }
        if searchResults.count == 0 && !localSearch.isSearching {
            updateNearby()
        }
    }

    // MARK: - UISearchBarDelegate

    func searchBarShouldBeginEditing(_ searchBar: UISearchBar) -> Bool {
        let cancelItem = UIBarButtonItem.init(barButtonSystemItem: .cancel, target: self, action: #selector(cancelTapped))
        navigationItem.setRightBarButton(cancelItem, animated: true)
        return true
    }

    func searchBarShouldEndEditing(_ searchBar: UISearchBar) -> Bool {
        navigationItem.setRightBarButton(nil, animated: true)
        return true
    }

    func searchBarSearchButtonClicked(_ searchBar: UISearchBar) {
        updateNearby()
    }

    func searchBar(_ searchBar: UISearchBar, textDidChange searchText: String) {
        if let timer = autoUpdateTimer {
            timer.invalidate()
            autoUpdateTimer = nil
        }

        // automatically update search results 1 second after you finish typing, even if you didn't hit the Search button
        autoUpdateTimer = Timer.scheduledTimer(timeInterval: 1, target: self, selector: #selector(updateNearby), userInfo: nil, repeats: false)
    }

    // MARK: - UITableViewController

    override func scrollViewWillBeginDragging(_ scrollView: UIScrollView) {
        searchBar.resignFirstResponder()
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return searchResults.count
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let reuseIdentifier = "MRNearbyPlacemarkCell"
        var cell: UITableViewCell
        if let dequeueCell = tableView.dequeueReusableCell(withIdentifier: reuseIdentifier) {
            cell = dequeueCell
        } else {
            cell = UITableViewCell(style: .value1, reuseIdentifier: reuseIdentifier)
        }
        if indexPath.row < searchResults.count {
            let result = searchResults[indexPath.row]
            if let placemarkName = result.placemark.name, placemarkName.count > 0 {
                cell.textLabel?.text = placemarkName
            } else if let typeName = result.placemark.typeName, typeName.count > 0 {
                cell.textLabel?.text = typeName
            } else {
                cell.textLabel?.text = "Placemark"
            }
            if result.expectedTravelTime != 0 {
                cell.detailTextLabel?.text = timeStringFor(interval: result.expectedTravelTime)
            } else {
                cell.detailTextLabel?.text = ""
            }

            cell.accessoryType = .disclosureIndicator
        }
        return cell
    }

    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        if indexPath.row < searchResults.count {
            presentDirections(to: searchResults[indexPath.row].placemark)
        }
    }

    // MARK: - Actions

    @objc private func updateNearby() {
        // a valid location is required for local search
        guard let currentLocation = locationManager.location else {
            refreshControl?.endRefreshing()
            return
        }

        // kill the auto-update timer if it was running
        autoUpdateTimer?.invalidate()
        autoUpdateTimer = nil

        refreshControl?.beginRefreshing()

        // configure a local search with our current location and a search term if appropriate
        let request = MRLocalSearchRequest()
        request.app = MREditorKey(identifier: MSExampleHost.appID())
        request.location = currentLocation
        request.transportType = .walking
        request.limit = 10
        request.naturalLanguageQuery = searchBar.text

        search = MRLocalSearch(request: request)
        search?.start(completionHandler: { [weak self] response, error in
            if let lastSearchResults = response?.results {
                guard let strongSelf = self else {
                    return
                }
                strongSelf.searchResults = lastSearchResults
                strongSelf.refreshControl?.endRefreshing()
                DispatchQueue.main.async {
                    strongSelf.tableView.reloadData()
                }
            } else if let lastError = error {
                print("Error loading search results: " + lastError.localizedDescription)
            }
        })
    }

    @objc private func cancelTapped() {
        searchBar.text = nil
        updateNearby()
        searchBar.resignFirstResponder()
    }

    // MARK: - Utility

    private func timeStringFor(interval: TimeInterval) -> String {
        if interval >= 60 {
            return String.init(format: "%0.0f min", interval / 60)
        } else {
            return String.init(format: "%0.0f sec", interval)
        }
    }
}
