//
//  MSSearchTableViewController.swift
//  MeridianSwiftSamples
//
//  Created by Stephen Kelly on 5/22/17.
//  Copyright © 2017 Aruba Networks. All rights reserved.
//

import UIKit

class MSSearchTableViewController: UITableViewController, UISearchBarDelegate, MSExampleViewController {
    private lazy var searchBar: UISearchBar = {
        let searchBar = UISearchBar(frame: CGRect(x: 0, y: 0, width: view.frame.size.width, height: 44))
        searchBar.delegate = self
        searchBar.placeholder = "Find placemarks, etc"
        return searchBar
    }()

    private var search: MRSearch?
    private var autoUpdateTimer: Timer?
    private var searchResults = [AnyObject]()

    override func viewDidLoad() {
        super.viewDidLoad()
        navigationItem.titleView = searchBar
    }

    // MARK: - MSExampleViewController

    static func exampleTitle() -> String {
        return "Search"
    }

    static func exampleDetail() -> String {
        return "Example of how to use the MRSearch API to search for Placemarks and Tags and display the results."
    }

    // MARK: - UISearchBarDelegate

    func searchBarShouldBeginEditing(_ searchBar: UISearchBar) -> Bool {
        let cancelItem = UIBarButtonItem.init(barButtonSystemItem: .cancel, target: self, action: #selector(cancelTapped))
        navigationItem.setRightBarButton(cancelItem, animated: true)
        return true
    }

    func searchBarShouldEndEditing(_ searchBar: UISearchBar) -> Bool {
        navigationItem.setRightBarButton(nil, animated: true)
        return true
    }

    func searchBarSearchButtonClicked(_ searchBar: UISearchBar) {
        startSearch()
    }

    func searchBar(_ searchBar: UISearchBar, textDidChange searchText: String) {
        if let timer = autoUpdateTimer {
            timer.invalidate()
            autoUpdateTimer = nil
        }

        // automatically update search results 1 second after you finish typing, even if you didn't hit the Search button
        autoUpdateTimer = Timer.scheduledTimer(timeInterval: 1, target: self, selector: #selector(startSearch), userInfo: nil, repeats: false)
    }

    override func scrollViewWillBeginDragging(_ scrollView: UIScrollView) {
        searchBar.resignFirstResponder()
    }

    // MARK: - UITableViewController

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return searchResults.count
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let reuseIdentifier = "MRSearchResultCell"
        var cell: UITableViewCell
        if let dequeueCell = tableView.dequeueReusableCell(withIdentifier: reuseIdentifier) {
            cell = dequeueCell
        } else {
            cell = UITableViewCell(style: .value1, reuseIdentifier: reuseIdentifier)
        }
        if indexPath.row < searchResults.count {
            let result = searchResults[indexPath.row]
            var name = "Unknown name"
            var kind = "Unknown kind"
            switch result {
            case let placemark as MRPlacemark:
                if let placemarkName = placemark.name, placemarkName.count > 0 {
                    name = placemarkName
                } else if let typeName = placemark.typeName, typeName.count > 0 {
                    name = typeName
                }
                kind = "Placemark"
            case let tag as MRTag:
                if let tagName = tag.name {
                    name = tagName
                }
                kind = "Tag"
            case let map as MRMap:
                name = map.name
                kind = "Map"
            default:
                break
            }
            cell.textLabel?.text = name
            cell.detailTextLabel?.text = kind
            cell.accessoryType = .disclosureIndicator
        }
        return cell
    }

    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        if indexPath.row < searchResults.count {
            switch searchResults[indexPath.row] {
            case let placemark as MRPlacemark:
                presentDirections(to: placemark)
            case let tag as MRTag:
                loadLocationAndPresentTag(tagMac: tag.mac)
            case let map as MRMap:
                let controller = MRMapViewController()
                controller.mapView.mapKey = MREditorKey(forMap: map.key.identifier, app: MSExampleHost.appID())
                navigationController?.pushViewController(controller, animated: true)
            default:
                break
            }
        }
        tableView.deselectRow(at: indexPath, animated: true)
    }

    // MARK: - Utilities

    func loadLocationAndPresentTag(tagMac: String) {
        // MRTag returned via search does not contain location data, so use the asset-beacons API to get more info
        // Future SDK enhancements should make this explicit URL creation unnecessary
        let config = MRConfig()
        let tagEndpoint = config.editorURL() + "/api/locations/\(MSExampleHost.appID())/asset-beacons/\(tagMac)"
        guard let url = URL(string: tagEndpoint) else {
            print("Error: cannot create URL")
            return
        }

        let task = URLSession.shared.dataTask(with: URLRequest(url: url)) {data, response, error in
            if let requestError = error {
                print("Tag request error: \(requestError.localizedDescription)")
                return
            }

            guard let tagResponse = response as? HTTPURLResponse, let tagData = data else {
                print("Error: did not receive tag response")
                return
            }

            if tagResponse.statusCode != 200 {
                print("Error: HTTP status code %li", tagResponse.statusCode)
                return
            }

            do {
                guard let json = try JSONSerialization.jsonObject(with: tagData, options: []) as? [String: Any] else {
                        print("error trying to convert data to JSON")
                        return
                }

                // Pull up the last map that tag was seen on
                guard let calculationDictionary = json["calculations"] as? [String: Any],
                    let defaultDictionary = calculationDictionary["default"] as? [String: Any],
                    let locationDictionary = defaultDictionary["location"] as? [String: Any],
                    let mapID = locationDictionary["map_id"] as? String
                else {
                    print("Unable to retrieve location info from tag response")
                    return
                }

                let controller = MRMapViewController()
                controller.mapView.mapKey = MREditorKey(forMap: mapID, app: MSExampleHost.appID())
                controller.mapView.showsTags = true
                controller.mapView.tagPredicate = NSPredicate.init(format: "mac LIKE %@", tagMac)
                DispatchQueue.main.async {
                    self.navigationController?.pushViewController(controller, animated: true)
                }
            } catch {
                print("error trying to convert data to JSON")
                return
            }
        }
        task.resume()
    }

    // MARK: - Actions

    @objc private func startSearch() {
        if let text = searchBar.text, text.count > 0 {
            let searchRequest = MRSearchRequest()
            searchRequest.app = MREditorKey(identifier: MSExampleHost.appID())
            searchRequest.naturalLanguageQuery = text

            search = MRSearch.init(request: searchRequest)
            searchResults.removeAll()
            search?.start(completionHandler: { [weak self] results, error in
                if let lastSearchResults = results {
                    guard let strongSelf = self else {
                        return
                    }
                    strongSelf.searchResults += lastSearchResults as [AnyObject]
                    DispatchQueue.main.async {
                        strongSelf.tableView.reloadData()
                    }
                } else if let lastError = error {
                    print("Error loading search results: " + lastError.localizedDescription)
                }}, finishedHandler: { [weak self] in
                    print("Search Complete")
                    self?.search = nil
            })

        } else {
            search?.cancel()
            searchResults.removeAll()
            tableView.reloadData()
        }
    }

    @objc private func cancelTapped() {
        searchBar.text = nil
        searchBar.resignFirstResponder()
        startSearch()
    }
}
